use argh::FromArgs;

use crate::action::ActionExec;
use crate::api::ApiClient;
use crate::config::Config;
use crate::error::Error;

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "get")]
/// get info about a specific torrent on qBittorrent
pub struct GetAction {
    #[argh(switch)]
    /// the positional argument is a magnet link, not an infohash
    magnet: bool,
    #[argh(positional)]
    /// the infohash (or magnet link with --magnet) to retrieve
    torrent: String,
}

impl ActionExec for GetAction {
    fn exec(&self, config: &Config) -> Result<(), Error> {
        let api = ApiClient::from_config(&config)?;
        let res = api.get(&self.torrent)?;
        if let Some(s) = res {
            println!("{}", s)
        }
        Ok(())
    }
}
